/* Generated: Fri Feb 14 2020 13:24:52 GMT-0700 (Mountain Standard Time) */

#include "device/dataadapterdevice.h"
static void CCONV PhidgetDataAdapter_errorHandler(PhidgetChannelHandle ch, Phidget_ErrorEventCode code);
static PhidgetReturnCode CCONV PhidgetDataAdapter_bridgeInput(PhidgetChannelHandle phid,
  BridgePacket *bp);
static PhidgetReturnCode CCONV PhidgetDataAdapter_setStatus(PhidgetChannelHandle phid,
  BridgePacket *bp);
static PhidgetReturnCode CCONV PhidgetDataAdapter_getStatus(PhidgetChannelHandle phid,
  BridgePacket **bp);
static PhidgetReturnCode CCONV PhidgetDataAdapter_initAfterOpen(PhidgetChannelHandle phid);
static PhidgetReturnCode CCONV PhidgetDataAdapter_setDefaults(PhidgetChannelHandle phid);
static void CCONV PhidgetDataAdapter_fireInitialEvents(PhidgetChannelHandle phid);
static int CCONV PhidgetDataAdapter_hasInitialState(PhidgetChannelHandle phid);

struct _PhidgetDataAdapter {
	struct _PhidgetChannel phid;
	uint32_t baudRate;
	uint32_t minBaudRate;
	uint32_t maxBaudRate;
	uint32_t dataBits;
	uint32_t minDataBits;
	uint32_t maxDataBits;
	PhidgetDataAdapter_RTSMode handshakeMode;
	uint32_t maxPacketLength;
	PhidgetDataAdapter_Parity parity;
	PhidgetDataAdapter_Protocol protocol;
	PhidgetDataAdapter_StopBits stopBits;
	PhidgetDataAdapter_OnPacketCallback Packet;
	void *PacketCtx;
};

static PhidgetReturnCode CCONV
_setStatus(PhidgetChannelHandle phid, BridgePacket *bp) {
	PhidgetDataAdapterHandle ch;
	int version;

	ch = (PhidgetDataAdapterHandle)phid;

	version = getBridgePacketUInt32ByName(bp, "_class_version_");
	if (version != 1) {
		loginfo("%"PRIphid": server/client class version mismatch: %d != 1 - functionality may be limited.", phid, version);
	}

	if(version >= 3)
		ch->baudRate = getBridgePacketUInt32ByName(bp, "baudRate");
	if(version >= 3)
		ch->minBaudRate = getBridgePacketUInt32ByName(bp, "minBaudRate");
	if(version >= 3)
		ch->maxBaudRate = getBridgePacketUInt32ByName(bp, "maxBaudRate");
	if(version >= 3)
		ch->dataBits = getBridgePacketUInt32ByName(bp, "dataBits");
	if(version >= 3)
		ch->minDataBits = getBridgePacketUInt32ByName(bp, "minDataBits");
	if(version >= 3)
		ch->maxDataBits = getBridgePacketUInt32ByName(bp, "maxDataBits");
	if(version >= 3)
		ch->handshakeMode = getBridgePacketInt32ByName(bp, "handshakeMode");
	if(version >= 3)
		ch->maxPacketLength = getBridgePacketUInt32ByName(bp, "maxPacketLength");
	if(version >= 3)
		ch->parity = getBridgePacketInt32ByName(bp, "parity");
	if(version >= 3)
		ch->protocol = getBridgePacketInt32ByName(bp, "protocol");
	if(version >= 3)
		ch->stopBits = getBridgePacketInt32ByName(bp, "stopBits");

	return (EPHIDGET_OK);
}

static PhidgetReturnCode CCONV
_getStatus(PhidgetChannelHandle phid, BridgePacket **bp) {
	PhidgetDataAdapterHandle ch;

	ch = (PhidgetDataAdapterHandle)phid;

	return (createBridgePacket(bp, 0, "_class_version_=%u"
	  ",baudRate=%u"
	  ",minBaudRate=%u"
	  ",maxBaudRate=%u"
	  ",dataBits=%u"
	  ",minDataBits=%u"
	  ",maxDataBits=%u"
	  ",handshakeMode=%d"
	  ",maxPacketLength=%u"
	  ",parity=%d"
	  ",protocol=%d"
	  ",stopBits=%d"
	  ,1 /* class version */
	  ,ch->baudRate
	  ,ch->minBaudRate
	  ,ch->maxBaudRate
	  ,ch->dataBits
	  ,ch->minDataBits
	  ,ch->maxDataBits
	  ,ch->handshakeMode
	  ,ch->maxPacketLength
	  ,ch->parity
	  ,ch->protocol
	  ,ch->stopBits
	));
}

static PhidgetReturnCode CCONV
_bridgeInput(PhidgetChannelHandle phid, BridgePacket *bp) {
	PhidgetDataAdapterHandle ch;
	PhidgetReturnCode res;

	ch = (PhidgetDataAdapterHandle)phid;
	res = EPHIDGET_OK;

	switch (bp->vpkt) {
	case BP_DATAOUT:
		res = DEVBRIDGEINPUT(phid, bp);
		break;
	case BP_SETBAUDRATE:
		TESTRANGE_IOP(bp->iop, "%"PRIu32, getBridgePacketUInt32(bp, 0), ch->minBaudRate,
		  ch->maxBaudRate);
		res = DEVBRIDGEINPUT(phid, bp);
		if (res != EPHIDGET_OK) {
			break;
		}
		ch->baudRate = getBridgePacketUInt32(bp, 0);
		if (bridgePacketIsFromNet(bp))
			FIRE_PROPERTYCHANGE(ch, "BaudRate");
		break;
	case BP_SETDATABITS:
		TESTRANGE_IOP(bp->iop, "%"PRIu32, getBridgePacketUInt32(bp, 0), ch->minDataBits,
		  ch->maxDataBits);
		res = DEVBRIDGEINPUT(phid, bp);
		if (res != EPHIDGET_OK) {
			break;
		}
		ch->dataBits = getBridgePacketUInt32(bp, 0);
		if (bridgePacketIsFromNet(bp))
			FIRE_PROPERTYCHANGE(ch, "DataBits");
		break;
	case BP_SETHANDSHAKEMODE:
		if (!supportedRTS_CTS_Mode(phid, (PhidgetDataAdapter_RTSMode)getBridgePacketInt32(bp, 0)))
			return (MOS_ERROR(bp->iop, EPHIDGET_INVALIDARG,
			  "Specified RTS_CTS_Mode is unsupported by this device."));
		res = DEVBRIDGEINPUT(phid, bp);
		if (res != EPHIDGET_OK) {
			break;
		}
		ch->handshakeMode = getBridgePacketInt32(bp, 0);
		if (bridgePacketIsFromNet(bp))
			FIRE_PROPERTYCHANGE(ch, "HandshakeMode");
		break;
	case BP_SETPARITY:
		if (!supportedParity(phid, (PhidgetDataAdapter_Parity)getBridgePacketInt32(bp, 0)))
			return (MOS_ERROR(bp->iop, EPHIDGET_INVALIDARG,
			  "Specified Parity is unsupported by this device."));
		res = DEVBRIDGEINPUT(phid, bp);
		if (res != EPHIDGET_OK) {
			break;
		}
		ch->parity = getBridgePacketInt32(bp, 0);
		if (bridgePacketIsFromNet(bp))
			FIRE_PROPERTYCHANGE(ch, "Parity");
		break;
	case BP_SETPROTOCOL:
		if (!supportedProtocol(phid, (PhidgetDataAdapter_Protocol)getBridgePacketInt32(bp, 0)))
			return (MOS_ERROR(bp->iop, EPHIDGET_INVALIDARG,
			  "Specified Protocol is unsupported by this device."));
		res = DEVBRIDGEINPUT(phid, bp);
		if (res != EPHIDGET_OK) {
			break;
		}
		ch->protocol = getBridgePacketInt32(bp, 0);
		if (bridgePacketIsFromNet(bp))
			FIRE_PROPERTYCHANGE(ch, "Protocol");
		break;
	case BP_SETSTOPBITS:
		if (!supportedStopBits(phid, (PhidgetDataAdapter_StopBits)getBridgePacketInt32(bp, 0)))
			return (MOS_ERROR(bp->iop, EPHIDGET_INVALIDARG,
			  "Specified StopBits is unsupported by this device."));
		res = DEVBRIDGEINPUT(phid, bp);
		if (res != EPHIDGET_OK) {
			break;
		}
		ch->stopBits = getBridgePacketInt32(bp, 0);
		if (bridgePacketIsFromNet(bp))
			FIRE_PROPERTYCHANGE(ch, "StopBits");
		break;
	default:
		logerr("%"PRIphid": unsupported bridge packet:0x%x", phid, bp->vpkt);
		res = EPHIDGET_UNSUPPORTED;
	}

	return (res);
}

static PhidgetReturnCode CCONV
_initAfterOpen(PhidgetChannelHandle phid) {
	PhidgetDataAdapterHandle ch;
	PhidgetReturnCode ret;

	TESTPTR(phid);
	ch = (PhidgetDataAdapterHandle)phid;

	ret = EPHIDGET_OK;


	switch (phid->UCD->uid) {
	case PHIDCHUID_ADP1001_DATAADAPTER_100:
		ch->maxPacketLength = 48;
		break;
	case PHIDCHUID_ADP_RS232_DATAADAPTER_100:
		ch->maxPacketLength = 1024;
		ch->baudRate = 9600;
		ch->minBaudRate = 800;
		ch->maxBaudRate = 120000;
		ch->parity = PARITY_MODE_NONE;
		ch->stopBits = STOP_BITS_ONE;
		ch->dataBits = 8;
		ch->minDataBits = 7;
		ch->maxDataBits = 8;
		ch->handshakeMode = RTS_CTS_MODE_READY_TO_RECEIVE;
		break;
	case PHIDCHUID_ADP_RS485_422_DATAADAPTER_100:
		ch->maxPacketLength = 1024;
		ch->baudRate = 9600;
		ch->minBaudRate = 800;
		ch->maxBaudRate = 2500000;
		ch->parity = PARITY_MODE_NONE;
		ch->stopBits = STOP_BITS_ONE;
		ch->dataBits = 8;
		ch->minDataBits = 7;
		ch->maxDataBits = 8;
		ch->protocol = PROTOCOL_RS485;
		break;
	default:
		MOS_PANIC("Unsupported Channel");
	}


	return (ret);
}

static PhidgetReturnCode CCONV
_setDefaults(PhidgetChannelHandle phid) {
	PhidgetDataAdapterHandle ch;
	PhidgetReturnCode ret;

	TESTPTR(phid);

	ch = (PhidgetDataAdapterHandle)phid;
	ret = EPHIDGET_OK;

	switch (phid->UCD->uid) {
	case PHIDCHUID_ADP1001_DATAADAPTER_100:
		break;
	case PHIDCHUID_ADP_RS232_DATAADAPTER_100:
		ret = bridgeSendToDevice(phid, BP_SETBAUDRATE, NULL, NULL, "%u", ch->baudRate);
		if (ret != EPHIDGET_OK) {
			break;
		}
		break;
	case PHIDCHUID_ADP_RS485_422_DATAADAPTER_100:
		ret = bridgeSendToDevice(phid, BP_SETBAUDRATE, NULL, NULL, "%u", ch->baudRate);
		if (ret != EPHIDGET_OK) {
			break;
		}
		break;
	default:
		MOS_PANIC("Unsupported Channel");
	}

	return (ret);
}

static void CCONV
_fireInitialEvents(PhidgetChannelHandle phid) {

}

static int CCONV
_hasInitialState(PhidgetChannelHandle phid) {

	return (PTRUE);
}

static void CCONV
PhidgetDataAdapter_free(PhidgetChannelHandle *ch) {

	mos_free(*ch, sizeof (struct _PhidgetDataAdapter));
}

API_PRETURN
PhidgetDataAdapter_create(PhidgetDataAdapterHandle *phidp) {

	CHANNELCREATE_BODY(DataAdapter, PHIDCHCLASS_DATAADAPTER);
	return (EPHIDGET_OK);
}

API_PRETURN
PhidgetDataAdapter_delete(PhidgetDataAdapterHandle *phidp) {

	return (Phidget_delete((PhidgetHandle *)phidp));
}

API_PRETURN
PhidgetDataAdapter_setBaudRate(PhidgetDataAdapterHandle ch, uint32_t baudRate) {

	TESTPTR_PR(ch);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_DATAADAPTER);
	TESTATTACHED_PR(ch);

	return (bridgeSendToDevice((PhidgetChannelHandle)ch, BP_SETBAUDRATE, NULL, NULL, "%u", baudRate));
}

API_PRETURN
PhidgetDataAdapter_getBaudRate(PhidgetDataAdapterHandle ch, uint32_t *baudRate) {

	TESTPTR_PR(ch);
	TESTPTR_PR(baudRate);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_DATAADAPTER);
	TESTATTACHED_PR(ch);

	switch (ch->phid.UCD->uid) {
	case PHIDCHUID_ADP1001_DATAADAPTER_100:
		return (PHID_RETURN(EPHIDGET_UNSUPPORTED));
	default:
		break;
	}

	*baudRate = ch->baudRate;
	if (ch->baudRate == (uint32_t)PUNK_UINT32)
		return (PHID_RETURN(EPHIDGET_UNKNOWNVAL));
	return (EPHIDGET_OK);
}

API_PRETURN
PhidgetDataAdapter_getMinBaudRate(PhidgetDataAdapterHandle ch, uint32_t *minBaudRate) {

	TESTPTR_PR(ch);
	TESTPTR_PR(minBaudRate);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_DATAADAPTER);
	TESTATTACHED_PR(ch);

	switch (ch->phid.UCD->uid) {
	case PHIDCHUID_ADP1001_DATAADAPTER_100:
		return (PHID_RETURN(EPHIDGET_UNSUPPORTED));
	default:
		break;
	}

	*minBaudRate = ch->minBaudRate;
	if (ch->minBaudRate == (uint32_t)PUNK_UINT32)
		return (PHID_RETURN(EPHIDGET_UNKNOWNVAL));
	return (EPHIDGET_OK);
}

API_PRETURN
PhidgetDataAdapter_getMaxBaudRate(PhidgetDataAdapterHandle ch, uint32_t *maxBaudRate) {

	TESTPTR_PR(ch);
	TESTPTR_PR(maxBaudRate);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_DATAADAPTER);
	TESTATTACHED_PR(ch);

	switch (ch->phid.UCD->uid) {
	case PHIDCHUID_ADP1001_DATAADAPTER_100:
		return (PHID_RETURN(EPHIDGET_UNSUPPORTED));
	default:
		break;
	}

	*maxBaudRate = ch->maxBaudRate;
	if (ch->maxBaudRate == (uint32_t)PUNK_UINT32)
		return (PHID_RETURN(EPHIDGET_UNKNOWNVAL));
	return (EPHIDGET_OK);
}

API_PRETURN
PhidgetDataAdapter_setDataBits(PhidgetDataAdapterHandle ch, uint32_t dataBits) {

	TESTPTR_PR(ch);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_DATAADAPTER);
	TESTATTACHED_PR(ch);

	return (bridgeSendToDevice((PhidgetChannelHandle)ch, BP_SETDATABITS, NULL, NULL, "%u", dataBits));
}

API_PRETURN
PhidgetDataAdapter_getDataBits(PhidgetDataAdapterHandle ch, uint32_t *dataBits) {

	TESTPTR_PR(ch);
	TESTPTR_PR(dataBits);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_DATAADAPTER);
	TESTATTACHED_PR(ch);

	switch (ch->phid.UCD->uid) {
	case PHIDCHUID_ADP1001_DATAADAPTER_100:
		return (PHID_RETURN(EPHIDGET_UNSUPPORTED));
	default:
		break;
	}

	*dataBits = ch->dataBits;
	if (ch->dataBits == (uint32_t)PUNK_UINT32)
		return (PHID_RETURN(EPHIDGET_UNKNOWNVAL));
	return (EPHIDGET_OK);
}

API_PRETURN
PhidgetDataAdapter_getMinDataBits(PhidgetDataAdapterHandle ch, uint32_t *minDataBits) {

	TESTPTR_PR(ch);
	TESTPTR_PR(minDataBits);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_DATAADAPTER);
	TESTATTACHED_PR(ch);

	switch (ch->phid.UCD->uid) {
	case PHIDCHUID_ADP1001_DATAADAPTER_100:
		return (PHID_RETURN(EPHIDGET_UNSUPPORTED));
	default:
		break;
	}

	*minDataBits = ch->minDataBits;
	if (ch->minDataBits == (uint32_t)PUNK_UINT32)
		return (PHID_RETURN(EPHIDGET_UNKNOWNVAL));
	return (EPHIDGET_OK);
}

API_PRETURN
PhidgetDataAdapter_getMaxDataBits(PhidgetDataAdapterHandle ch, uint32_t *maxDataBits) {

	TESTPTR_PR(ch);
	TESTPTR_PR(maxDataBits);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_DATAADAPTER);
	TESTATTACHED_PR(ch);

	switch (ch->phid.UCD->uid) {
	case PHIDCHUID_ADP1001_DATAADAPTER_100:
		return (PHID_RETURN(EPHIDGET_UNSUPPORTED));
	default:
		break;
	}

	*maxDataBits = ch->maxDataBits;
	if (ch->maxDataBits == (uint32_t)PUNK_UINT32)
		return (PHID_RETURN(EPHIDGET_UNKNOWNVAL));
	return (EPHIDGET_OK);
}

API_PRETURN
PhidgetDataAdapter_setHandshakeMode(PhidgetDataAdapterHandle ch,
  PhidgetDataAdapter_RTSMode handshakeMode) {

	TESTPTR_PR(ch);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_DATAADAPTER);
	TESTATTACHED_PR(ch);

	return (bridgeSendToDevice((PhidgetChannelHandle)ch, BP_SETHANDSHAKEMODE, NULL, NULL, "%d",
	  handshakeMode));
}

API_PRETURN
PhidgetDataAdapter_getHandshakeMode(PhidgetDataAdapterHandle ch,
  PhidgetDataAdapter_RTSMode *handshakeMode) {

	TESTPTR_PR(ch);
	TESTPTR_PR(handshakeMode);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_DATAADAPTER);
	TESTATTACHED_PR(ch);

	switch (ch->phid.UCD->uid) {
	case PHIDCHUID_ADP1001_DATAADAPTER_100:
	case PHIDCHUID_ADP_RS485_422_DATAADAPTER_100:
		return (PHID_RETURN(EPHIDGET_UNSUPPORTED));
	default:
		break;
	}

	*handshakeMode = ch->handshakeMode;
	if (ch->handshakeMode == (PhidgetDataAdapter_RTSMode)PUNK_ENUM)
		return (PHID_RETURN(EPHIDGET_UNKNOWNVAL));
	return (EPHIDGET_OK);
}

API_PRETURN
PhidgetDataAdapter_getMaxPacketLength(PhidgetDataAdapterHandle ch, uint32_t *maxPacketLength) {

	TESTPTR_PR(ch);
	TESTPTR_PR(maxPacketLength);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_DATAADAPTER);
	TESTATTACHED_PR(ch);

	*maxPacketLength = ch->maxPacketLength;
	if (ch->maxPacketLength == (uint32_t)PUNK_UINT32)
		return (PHID_RETURN(EPHIDGET_UNKNOWNVAL));
	return (EPHIDGET_OK);
}

API_PRETURN
PhidgetDataAdapter_setParity(PhidgetDataAdapterHandle ch, PhidgetDataAdapter_Parity parity) {

	TESTPTR_PR(ch);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_DATAADAPTER);
	TESTATTACHED_PR(ch);

	return (bridgeSendToDevice((PhidgetChannelHandle)ch, BP_SETPARITY, NULL, NULL, "%d", parity));
}

API_PRETURN
PhidgetDataAdapter_getParity(PhidgetDataAdapterHandle ch, PhidgetDataAdapter_Parity *parity) {

	TESTPTR_PR(ch);
	TESTPTR_PR(parity);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_DATAADAPTER);
	TESTATTACHED_PR(ch);

	switch (ch->phid.UCD->uid) {
	case PHIDCHUID_ADP1001_DATAADAPTER_100:
		return (PHID_RETURN(EPHIDGET_UNSUPPORTED));
	default:
		break;
	}

	*parity = ch->parity;
	if (ch->parity == (PhidgetDataAdapter_Parity)PUNK_ENUM)
		return (PHID_RETURN(EPHIDGET_UNKNOWNVAL));
	return (EPHIDGET_OK);
}

API_PRETURN
PhidgetDataAdapter_setProtocol(PhidgetDataAdapterHandle ch, PhidgetDataAdapter_Protocol protocol) {

	TESTPTR_PR(ch);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_DATAADAPTER);
	TESTATTACHED_PR(ch);

	return (bridgeSendToDevice((PhidgetChannelHandle)ch, BP_SETPROTOCOL, NULL, NULL, "%d", protocol));
}

API_PRETURN
PhidgetDataAdapter_getProtocol(PhidgetDataAdapterHandle ch, PhidgetDataAdapter_Protocol *protocol) {

	TESTPTR_PR(ch);
	TESTPTR_PR(protocol);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_DATAADAPTER);
	TESTATTACHED_PR(ch);

	switch (ch->phid.UCD->uid) {
	case PHIDCHUID_ADP1001_DATAADAPTER_100:
	case PHIDCHUID_ADP_RS232_DATAADAPTER_100:
		return (PHID_RETURN(EPHIDGET_UNSUPPORTED));
	default:
		break;
	}

	*protocol = ch->protocol;
	if (ch->protocol == (PhidgetDataAdapter_Protocol)PUNK_ENUM)
		return (PHID_RETURN(EPHIDGET_UNKNOWNVAL));
	return (EPHIDGET_OK);
}

API_PRETURN
PhidgetDataAdapter_setStopBits(PhidgetDataAdapterHandle ch, PhidgetDataAdapter_StopBits stopBits) {

	TESTPTR_PR(ch);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_DATAADAPTER);
	TESTATTACHED_PR(ch);

	return (bridgeSendToDevice((PhidgetChannelHandle)ch, BP_SETSTOPBITS, NULL, NULL, "%d", stopBits));
}

API_PRETURN
PhidgetDataAdapter_getStopBits(PhidgetDataAdapterHandle ch, PhidgetDataAdapter_StopBits *stopBits) {

	TESTPTR_PR(ch);
	TESTPTR_PR(stopBits);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_DATAADAPTER);
	TESTATTACHED_PR(ch);

	switch (ch->phid.UCD->uid) {
	case PHIDCHUID_ADP1001_DATAADAPTER_100:
		return (PHID_RETURN(EPHIDGET_UNSUPPORTED));
	default:
		break;
	}

	*stopBits = ch->stopBits;
	if (ch->stopBits == (PhidgetDataAdapter_StopBits)PUNK_ENUM)
		return (PHID_RETURN(EPHIDGET_UNKNOWNVAL));
	return (EPHIDGET_OK);
}

API_PRETURN
PhidgetDataAdapter_setOnPacketHandler(PhidgetDataAdapterHandle ch,
  PhidgetDataAdapter_OnPacketCallback fptr, void *ctx) {

	TESTPTR_PR(ch);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_DATAADAPTER);

	ch->Packet = fptr;
	ch->PacketCtx = ctx;

	return (EPHIDGET_OK);
}
